#############################################################################
#
# Package: Absorbg procedure
#
# Language: Maple 6, 7, 8, 9.
#
# Description: Perfoms the contraction of the metric tensor.
#
# Developer: Renato Portugal
#            Laboratorio Nacional de Computacao Cientifica (LNCC)
#
# Web page: http://www.cbpf.br/~portugal/Canon.html
#
# Contact and bug report: Portugal@Lncc.Br
#
#############################################################################


`Absorbg/version` := proc()
options `Copyright 2003 by R Portugal. All rights reserved`;
  "Canon version 1.0, Jun, 16, 2003, Maple 6,7,8,9"
end proc:


#
# Global variable
#
SpaceDimension := 4:

#
# raises or lowers indices contrated by the metric
# it is assumed that the metric is symmetric
# The order of summed indices are not changed except the metric indices
#
Absorbg := proc(expr1)
local F1,expr;
options `Copyright 1998-2003 by R Portugal All rights reserved`;
expr := Canon[Canonical](eval(`Canon/Absorbg`(CanonUnPrint(expr1))));

# simplify Riemann to Ricci or Ricci scalar
F1 := proc(x)
local L;
L := `Canon/indices`(x);
if L[1]=[] then
     x
elif L[2]=[] then
     if op(1,x)=-op(3,x) then
          R[]
     else -R[]
     end if;
elif op(2,x)=-op(4,x) then
     R[op(L[2])]
else x
end if;
end proc;

expr := subs(map(x->x=F1(x),indets(expr,
             'R[anything,anything,anything,anything]')),expr);

# simplify Ricci to Ricci scalar
F1 := proc(x)
local L;
L := `Canon/indices`(x);
if L[1]=[] then
     x
elif L[2]=[] then
     R[]
else x
end if;
end proc;

CanonPrint(subs(map(x->x=F1(x),indets(expr,'R[anything,anything]')),expr));

end proc:



`Canon/Absorbg` := proc(expr)
local p1, i, L1, new_expr, T, ind1, terms_with_i, t1, j, t2, _t1, T2, X;
global g;

options `Copyright 1998-2003 by R Portugal All rights reserved`;

if not has(expr,g) then RETURN(expr) fi;

# calculate the trace
if type(expr,'g[anything,anything]') 
  then if op(1,expr)=-op(2,expr)
         then SpaceDimension
       else expr
       fi

elif type(expr,'{`+`,`^`,function}') 
  then map(procname,expr)

elif type(expr,`*`) 
  then T:=map(procname,expr);
       ind1:=op(1,`Canon/indices`(T));

       # no contraction with g
       if ind1=[] then RETURN(T) fi;

       for i in ind1 do
         terms_with_i := select(has,T,i);
         if not has(terms_with_i,g) or 
              not has(indets(terms_with_i,'g[anything,anything]'),i) or
                 not type(terms_with_i,`*`)
           then next
         fi;
         
         # 
         # absorb deltas
         #
         t1 := select(type,terms_with_i,
                    {g['contravariantindex','covariantindex'],
                     g['covariantindex','contravariantindex']});
         if t1<>1 and not type(t1,`*`)
           then j := op(subs(i=0,0=NULL,[op(t1)]));
                if has(terms_with_i,-i)
                  then T:=procname(
                             subs(i=`Canon/abs`(j),terms_with_i/t1))*
                                               (T/terms_with_i);
                  else T:=procname(subs(i=-j,terms_with_i/t1))*
                                               (T/terms_with_i);
                fi;
                next
         fi;

         if type(terms_with_i,`&*`('constanttensor,constanttensor'))
           then t1 := op(1,terms_with_i);
                t2 := op(2,terms_with_i);
                if op(0,t1)<>g
                  then _t1:=t1;
                       t1:=t2;
                       t2:=_t1;
                fi;
                T:=procname(subs(i=0,0=op(subs(i=0,0=NULL,t1)),t2))*
                                              (T/terms_with_i);
                if not type(T,`*`)
                  then RETURN(procname(T))
                fi

         elif type(op(1,terms_with_i),`+`) or
                     type(op(2,terms_with_i),`+`)
           then T:=procname(expand(terms_with_i))*(T/terms_with_i);
                if not type(T,`*`)
                  then RETURN(procname(T))
                fi
         fi;
       od;
       RETURN(T);
else expr
fi
end:


#
#
#
unprotect(_SCALAR):

#
# In principle _SCALAR is used in the Canon package as an inert function.
# Here we increase the robusteness of _SCALAR allowing any kind of 
# input. The user can use this function to input powers of scalar 
# expressions in a secure way.
#
# This version can be improved further.
#
# Example: (V[i]*V[-i])^2 is not allowed (free index in power).
#          _SCALAR(V[i]*V[-i])^2 is fine.
#
_SCALAR := proc(x)
options remember,system,`Copyright 2003 by R Portugal. All rights reserved`;
`Canon/CanonPrint`(`_SCALAR/1`(`Canon/CanonUnPrint`(x)))
end proc:


protect('_SCALAR');


`_SCALAR/1` := proc(x)
local L,terms2,terms1,terms0;
options `Copyright 2003 by R Portugal. All rights reserved`;
if type(x,'_SCALAR(anything)') then
     x
elif type(x,`*`) then
     L := `Canon/indices`(x);
     terms1,terms0 := 
          selectremove(
              proc(y)
               (type(y,'indexed^anything') and nops(op(1,y))>1) or 
                (type(y,indexed) and nops(y)>0 and not has(y,{op(L[2])})) and
                  not type(y,'scalarinvariant')
              end proc, x);
     if type(terms1,`*`) then
          terms1,terms2 := selectremove(type,terms1,'indexed^anything');
          if type(terms1,`*`) then 
               terms0*map(procname,terms1)*'_SCALAR'(terms2);
          else 
               terms0*procname(terms1)*'_SCALAR'(terms2);
          end if;

     elif type(terms1,`^`) then
          terms0*procname(terms1)
     else x
     end if;

elif type(x,'indexed^anything') and nops(op(1,x))>1 then
     '_SCALAR'(op(1,x))^op(2,x)

elif type(x,'{`+`,`^`,`=`,list,set}') then
     map('_SCALAR',x)
else
     x
end if;
end proc:


