#
# Test file for Canon
#
# Date: 05/Jun/2003
# Authors: L. R. U. Manssur and R. Portugal
#
#
# Conventions
#
# cor1,cor2,... are the correct results
# car1,car2,... are the calculations
#
# variables used:
#
#  p1,p2,... permutations
#  K1,K2,... generating sets
#  B1,B2,... bases
#  i1,i2,... points
#  sch1,sch2,... Schreier vectors
#  bac1,bac2,... backward pointers
#

#
# `Canon/prod`
#

p1 := [[1,2,3]]:
p2 := [[1,3,2]]:
cor1 := []:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-1`,cor1,car1)
fi;

p1 := [[1,2,3]]:
p2 := [-1,[[1,3,2]]]:
cor1 := [-1,[]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-2`,cor1,car1)
fi;

p1 := [[1,5,3],[4,10,15,11]]:
p2 := [[1,3,2],[11,15]]:
cor1 := [[1,5,2], [4,10,11]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-3`,cor1,car1)
fi;

p1 := [-1,[[1,5,3],[4,10,15,11]]]:
p2 := [-1,[[1,3,2],[11,15]]]:
cor1 := [[1,5,2], [4,10,11]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-4`,cor1,car1)
fi;

p1 := []:
p2 := []:
cor1 := []:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-5`,cor1,car1)
fi;

p1 := [-1,[]]:
p2 := [-1,[]]:
cor1 := []:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-6`,cor1,car1)
fi;

p1 := []:
p2 := [-1,[]]:
cor1 := [-1,[]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-7`,cor1,car1)
fi;

p1 := [-1,[]]:
p2 := []:
cor1 := [-1,[]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-8`,cor1,car1)
fi;

p1 := [-1,[]]:
p2 := [[1,2,3]]:
cor1 := [-1,[[1,2,3]]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-9`,cor1,car1)
fi;

p1 := [-1,[]]:
p2 := [-1,[[1,2,3]]]:
cor1 := [[1,2,3]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-10`,cor1,car1)
fi;

p1 := []:
p2 := [-1,[[1,2,3]]]:
cor1 := [-1,[[1,2,3]]]:
car1 := `Canon/prod`(p1,p2):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 1-11`,cor1,car1)
fi;



#
# `Canon/power`
#


i1 := 1:
p1 := []:
cor1 := 1:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-1`,cor1,car1)
fi;

i1 := 1:
p1 := [-1,[]]:
cor1 := 1:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-2`,cor1,car1)
fi;

i1 := 2:
p1 := [-1,[]]:
cor1 := 2:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-3`,cor1,car1)
fi;

i1 := 1:
p1 := [[1,3,2]]:
cor1 := 3:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-4`,cor1,car1)
fi;

i1 := 1:
p1 := [[1,3,2]]:
cor1 := 3:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-5`,cor1,car1)
fi;

i1 := 5:
p1 := [[2,6,5]]:
cor1 := 2:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-6`,cor1,car1)
fi;

i1 := 6:
p1 := [[1,3,2],[5,6]]:
cor1 := 5:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-7`,cor1,car1)
fi;

i1 := 1:
p1 := [-1,[[1,3,2]]]:
cor1 := 3:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-8`,cor1,car1)
fi;

i1 := 5:
p1 := [-1,[[2,6,5]]]:
cor1 := 2:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-9`,cor1,car1)
fi;

i1 := 6:
p1 := [-1,[[1,3,2],[5,6]]]:
cor1 := 5:
car1 := `Canon/power`(i1,p1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 2-10`,cor1,car1)
fi;




#
# `Canon/orbit` and `Canon/trace`
#



K1 := [[[1,4,5,11,6,10,3,2],[7,8]],
[[1,4,5,11,6,10,3,2],[8,9]],
[[1,4,2,3],[5,11,10,6]]]:
B1 := [1,2,3,4,5,6,7,8,9,10,11]:
i1 := 2:
cor1 := {1, 2, 3, 4, 5, 6, 10, 11}:
sch1:='sch1': bac1:='bac1':
car1 := `Canon/orbit1`(i1,K1,B1,'sch1','bac1'):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 3-1-1`,cor1,car1)
fi;

cor2 := table([11 = [[1,4,5,11,6,10,3,2],[7,8]],
1 = 0, 3 = [[1, 4, 2, 3], [5, 11, 10, 6]],
2= [[1,4,2,3],[5,11,10,6]], 4 = [[1,4,5,11,6,10,3,2],[7,8]], 
5 = [[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],
6 = [[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]], 
10 = [[1, 4, 2, 3], [5,11,10,6]]]):
car2 := eval(sch1):
if {indices(cor2)}<>{indices(car2)} or {entries(cor2)}<>{entries(car2)} then
ERROR(`file Canon`,`error n. 3-1-2`,eval(cor2),eval(car2))
fi;

cor3 := table([11 = 5, 1 = 0, 2=4, 3 = 2, 4 = 1, 5 = 4, 6 = 11, 10 = 11]):
car3 := bac1:
if {indices(cor3)}<>{indices(car3)} or {entries(cor3)}<>{entries(car3)} then
 ERROR(`file Canon`,`error n. 3-1-3`,eval(cor3),eval(car3))
fi;

K1 := [[[1,4,5,11,6,10,3,2],[7,8]],
[[1,4,5,11,6,10,3,2],[8,9]],
[[1,4,2,3],[5,11,10,6]]]:
B1 := [1,2,3,4,5,6,7,8,9,10,11]:
i1 := 1:
cor1 := {1, 2, 3, 4, 5, 6, 10, 11}:
sch1:='sch1': bac1:='bac1':
car1 := `Canon/orbit2`(i1,K1,B1,'sch1','bac1'):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 3-2-1`,cor1,car1)
fi;

cor2 := table([11 = [[1,4,5,11,6,10,3,2],[7,8]],
1 = 0, 3 = [[1, 4, 2, 3], [5, 11, 10, 6]],
2= [[1,4,2,3],[5,11,10,6]], 4 = [[1,4,5,11,6,10,3,2],[7,8]], 
5 = [[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],
6 = [[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]], 
10 = [[1, 4, 2, 3], [5,11,10,6]]]):
car2 := eval(sch1):
if {indices(cor2)}<>{indices(car2)} or {entries(cor2)}<>{entries(car2)} then
ERROR(`file Canon`,`error n. 3-2-2`,eval(cor2),eval(car2))
fi;

cor3 := table([11 = 5, 1 = 0, 2=4, 3 = 2, 4 = 1, 5 = 4, 6 = 11, 10 = 11]):
car3 := bac1:
if {indices(cor3)}<>{indices(car3)} or {entries(cor3)}<>{entries(car3)} then
 ERROR(`file Canon`,`error n. 3-2-3`,eval(cor3),eval(car3))
fi;

cor1 := [[1, 11, 3, 4, 6, 2, 5, 10], [7, 8]]:
car1 := `Canon/trace`(11,sch1,bac1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 3-2-4`,cor1,car1)
fi;

K1 := [[[1,4,5,11,6,10,3,2],[7,8]],
[[1,4,5,11,6,10,3,2],[8,9]],
[[1,4,2,3],[5,11,10,6]]]:
B1 := [1,2,3,4,5,6,7,8,9,10,11]:
i1 := 7:
cor1 := {7,8,9}:
sch1:='sch1': bac1:='bac1':
car1 := `Canon/orbit2`(i1,K1,B1,'sch1','bac1'):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 3-3-1`,cor1,car1)
fi;

cor2 := table([7 = 0, 
               8 = [[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]], 
               9 = [[1, 4, 5, 11, 6, 10, 3, 2], [8, 9]]]):
car2 := eval(sch1):
if {indices(cor2)}<>{indices(car2)} or {entries(cor2)}<>{entries(car2)} then
ERROR(`file Canon`,`error n. 3-3-2`,eval(cor2),eval(car2))
fi;

cor3 := table([(7)=0,(8)=7,(9)=8]):
car3 := bac1:
if {indices(cor3)}<>{indices(car3)} or {entries(cor3)}<>{entries(car3)} then
 ERROR(`file Canon`,`error n. 3-3-3`,eval(cor3),eval(car3))
fi;

cor1 := [[1, 5, 6, 3], [2, 4, 11, 10], [7, 9, 8]]:
car1 := `Canon/trace`(9,sch1,bac1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 3-3-4`,cor1,car1)
fi;



#
# `Canon/allorbits`
#



K1 := [[[1,4,5,11,6,10,3,2],[7,8]],
[[1,4,5,11,6,10,3,2],[8,9]],
[[1,4,2,3],[5,11,10,6]]]:
B1 := [1,2,3,4,5,6,7,8,9,10,11]:
cor1 := {{1, 2, 3, 4, 5, 6, 10, 11}, {7, 8, 9}}:
sch1:='sch1': bac1:='bac1':
car1 := `Canon/allorbits`(K1,B1,'sch1','bac1'):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 4-1-1`,cor1,car1)
fi;
cor2 := table([(11)=[[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],(1)=0,
(2)=[[1, 4, 2, 3], [5, 11, 10, 6]],(3)=[[1, 4, 2, 3], [5, 11, 10, 6]],
(4)=[[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],
(5)=[[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],
(6)=[[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],
(7)=0,(8)=[[1, 4, 5, 11, 6, 10, 3, 2], [7, 8]],
(9)=[[1, 4, 5, 11, 6, 10, 3, 2], [8, 9]],
(10)=[[1, 4, 2, 3], [5, 11, 10, 6]]]):
car2 := eval(sch1):
if {indices(cor2)}<>{indices(car2)} or {entries(cor2)}<>{entries(car2)} then
ERROR(`file Canon`,`error n. 4-1-2`,eval(cor2),eval(car2))
fi;
cor3 := table([(8)=7,(9)=8,(10)=11,(11)=5,(1)=0,(2)=4,(3)=2,(4)=1,(5)=4,
(6)=11,(7)=0]):
car3 := bac1:
if {indices(cor3)}<>{indices(car3)} or {entries(cor3)}<>{entries(car3)} then
 ERROR(`file Canon`,`error n. 4-1-3`,eval(cor3),eval(car3))
fi;



#
# `Canon/inv`
#


cor1:=[]:
car1:=`Canon/inv`([]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-1`,cor1,car1)
fi;

cor1:=[[1,2]]:
car1:=`Canon/inv`([[1,2]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-2`,cor1,car1)
fi;

cor1:=[-1,[[1,2]]]:
car1:=`Canon/inv`([-1,[[1,2]]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-3`,cor1,car1)
fi;

cor1:=[[1,2,4]]:
car1:=`Canon/inv`([[1,4,2]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-4`,cor1,car1)
fi;

cor1:=[[1,2,5],[3,8,7,9]]:
car1:=`Canon/inv`([[1,5,2],[3,9,7,8]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-5`,cor1,car1)
fi;

cor1:=[-1,[[1,2,4]]]:
car1:=`Canon/inv`([-1,[[1,4,2]]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-6`,cor1,car1)
fi;

cor1:=[-1,[[1,2,5],[3,8,7,9]]]:
car1:=`Canon/inv`([-1,[[1,5,2],[3,9,7,8]]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-7`,cor1,car1)
fi;

cor1:=[-1,[]]:
car1:=`Canon/inv`([-1,[]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-8`,cor1,car1)
fi;

cor1:=[-1,[[1,2,5],[3,8,7,9],[35,60]]]:
car1:=`Canon/inv`([-1,[[1,5,2],[3,9,7,8],[35,60]]]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 5-9`,cor1,car1)
fi;


#
# `Canon/min`
#

cor1:=NULL:
car1:=`Canon/min`({}):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 6-1`,cor1,car1)
fi;

cor1:=2:
car1:=`Canon/min`({2,3,4},[]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 6-2`,cor1,car1)
fi;

cor1:=4:
car1:=`Canon/min`({1,2,3,4},[4,3]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 6-3`,cor1,car1)
fi;


#
# `Canon/SortPermutations`
#

cor1:=[[[1,2]]]:
car1:=`Canon/SortPermutations`([[[1,2]]],[]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 7-1`,cor1,car1)
fi;

cor1:=[[[1,2]],[[2,3]],[[3,4]]]:
car1:=`Canon/SortPermutations`([[[2,3]],[[1,2]],[[3,4]]],[4,3,2,1]):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 7-2`,cor1,car1)
fi;


#
# Double coset canonical representative
#

K1 := {[-1,[[1,2]]],[[3,4]],[-1,[[5,6]]],[[1,5],[2,6]]}:
B1 := [1,3,6]:
p1 := [[1,3]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 12-1`,cor1,car1)
fi;

p1 := [-1,[[1,3]]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 12-2`,cor1,car1)
fi;

p1 := [[1,2]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 12-3`,cor1,car1)
fi;

p1 := [-1,[[1,2]]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 12-4`,cor1,car1)
fi;


#p1 := [[2, 3], [4, 5, 6]]:
#cor1 := [[2, 4, 5]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 12-5`,cor1,car1)
#fi;

#p1 := [-1,[[2, 3], [4, 5, 6]]]:
#cor1 := [-1,[[2, 4, 5]]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 12-6`,cor1,car1)
#fi;

#p1 := [[2, 4, 5]]:
#cor1 := [[2, 4, 5]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 12-7`,cor1,car1)
#fi;

#p1 := [-1,[[2, 4, 5]]]:
#cor1 := [-1,[[2, 4, 5]]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 12-8`,cor1,car1)
#fi;

#p1 := [[1, 5]]:
#cor1 := [-1, [[2, 5]]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 12-9`,cor1,car1)
#fi;

#p1 := [-1,[[1, 5]]]:
#cor1 := [[2, 5]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 12-10`,cor1,car1)
#fi;


K1 := {[-1,[[1,2]]],[[3,4]],[-1,[[5,6]]],[[1,5],[2,6]]}:
B1 := [1,2,3,4,5,6]:
p1 := [[1,3]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-1`,cor1,car1)
fi;

p1 := [-1,[[1,3]]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-2`,cor1,car1)
fi;

p1 := [[1,2]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-3`,cor1,car1)
fi;

p1 := [-1,[[1,2]]]:
cor1 := 0:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-4`,cor1,car1)
fi;


p1 := [[2, 3], [4, 5, 6]]:
cor1 := [-1, [[2, 3], [4, 5]]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-5`,cor1,car1)
fi;

p1 := [-1,[[2, 3], [4, 5, 6]]]:
cor1 := [[2, 3], [4, 5]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-6`,cor1,car1)
fi;

p1 := [[2, 4, 5]]:
cor1 := [-1, [[2, 3], [4, 5]]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-7`,cor1,car1)
fi;

p1 := [-1,[[2, 4, 5]]]:
cor1 := [[2, 3], [4, 5]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-8`,cor1,car1)
fi;

p1 := [[1, 5]]:
cor1 := [-1, [[2, 3, 5], [4, 6]]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-9`,cor1,car1)
fi;

p1 := [-1,[[1, 5]]]:
cor1 := [[2, 3, 5], [4, 6]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,3,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 13-10`,cor1,car1)
fi;


K1 := {[[1,2,4,5,7,3,6]],[[2,4],[3,5]],[[4,5],[6,7]],[[4,6],[5,7]]}:
B1 := [1,2,4]:
p1 := [[1,3]]:
cor1 := [[6,7]]:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,4,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 14-1`,cor1,car1)
fi;

p1 := [[1,3,2],[4,7,5]]:
cor1 := []:
car1 := `Canon/Canonical/g`(p1,K1,B1,0,4,1):
if cor1<>car1 then
ERROR(`file Canon`,`error n. 14-2`,cor1,car1)
fi;

#p1 := [[3, 5, 7], [6, 8]]:
#cor1 := [[3, 5, 7], [6, 8]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,4,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 14-3`,cor1,car1)
#fi;

#p1 := [[3, 8, 7, 5], [4, 6]]:
#cor1 := [[3, 5, 7], [6, 8]]:
#car1 := `Canon/Canonical/g`(p1,K1,B1,0,4,1):
#if cor1<>car1 then
#ERROR(`file Canon`,`error n. 14-4`,cor1,car1)
#fi;



#
# `Canon/isingroup`
#

K1:={[[1,2,4,5,7,3,6]],[[2,4],[3,5]], 
  [[2,6,3,7],[4,5]],[[4,7],[5,6]],[[4,5],[6,7]]}:
B1:=[1,2,4]:
p1:=[[1, 6, 2, 7, 4, 5, 3]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-1`,cor1,car1) 
fi;

K1:={[[1,2,4,5,7,3,6]],[[2,4],[3,5]], 
  [[2,6,3,7],[4,5]],[[4,7],[5,6]],[[4,5],[6,7]]}:
B1:=[1,2,4]:
p1:=[[1, 4, 6, 5, 2, 3, 7]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-2`,cor1,car1) 
fi;

K1:={[[1,2,4,5,7,3,6]],[[2,4],[3,5]], 
  [[2,6,3,7],[4,5]],[[4,7],[5,6]],[[4,5],[6,7]]}:
B1:=[1,2,4]:
p1:=[[1, 2, 3], [4, 5, 7]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-3`,cor1,car1) 
fi;

K1:={[[1,2,4,5,7,3,6]],[[2,4],[3,5]], 
  [[2,6,3,7],[4,5]],[[4,7],[5,6]],[[4,5],[6,7]]}:
B1:=[1,2,4]:
p1:=[[1, 3, 5], [2, 6, 4]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-4`,cor1,car1) 
fi;

K1:={[[1,2,4,5,7,3,6]],[[2,4],[3,5]], 
  [[2,6,3,7],[4,5]],[[4,7],[5,6]],[[4,5],[6,7]]}:
B1:=[1,2,4]:
p1:=[[1,2,3,4,5,6,7]]:
cor1:=false:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-5`,cor1,car1) 
fi;

K1:={[-1, [[1, 2]]], [-1, [[3, 4]]], 
    [-1, [[5, 6]]], [-1, [[7, 8]]], 
    [[1, 3], [2, 4]], [[5, 7], [6, 8]], 
    [[1, 5], [2, 6], [3, 7], [4, 8]]}:
B1:=[1,3,5,7]:
p1:=[[1,2]]:
cor1:=false:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-6`,cor1,car1) 
fi;

K1:={[-1, [[1, 2]]], [-1, [[3, 4]]], 
    [-1, [[5, 6]]], [-1, [[7, 8]]], 
    [[1, 3], [2, 4]], [[5, 7], [6, 8]], 
    [[1, 5], [2, 6], [3, 7], [4, 8]]}:
B1:=[1,3,5,7]:
p1:=[-1,[[1,2]]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-7`,cor1,car1) 
fi;

K1:={[-1, [[1, 2]]], [-1, [[3, 4]]], 
    [-1, [[5, 6]]], [-1, [[7, 8]]], 
    [[1, 3], [2, 4]], [[5, 7], [6, 8]], 
    [[1, 5], [2, 6], [3, 7], [4, 8]]}:
B1:=[1,3,5,7]:
p1:=[ -1,[[1,3],[2,4]] ]:
cor1:=false:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-8`,cor1,car1) 
fi;

K1:={[-1, [[1, 2]]], [-1, [[3, 4]]], 
    [-1, [[5, 6]]], [-1, [[7, 8]]], 
    [[1, 3], [2, 4]], [[5, 7], [6, 8]], 
    [[1, 5], [2, 6], [3, 7], [4, 8]]}:
B1:=[1,3,5,7]:
p1:=[[1,3],[2,4]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-9`,cor1,car1) 
fi;

K1:={[-1, [[1, 2]]], [-1, [[3, 4]]], 
    [-1, [[5, 6]]], [-1, [[7, 8]]], 
    [[1, 3], [2, 4]], [[5, 7], [6, 8]], 
    [[1, 5], [2, 6], [3, 7], [4, 8]]}:
B1:=[1,3,5,7]:
p1:=[[1,3],[2,4],[5,7],[6,8]]:
cor1:=true:
car1:=`Canon/isingroup`(p1,K1,B1):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 15-10`,cor1,car1) 
fi;


#
# `Canon/schreier-sims`
#

s:={[[1,2,4,5,7,3,6]],[[2,4],[3,5]]}:
cor1:={[[2,6,3,7],[4, 5]],[[1,2,4,5,7,3,6]],[[4,5],[6,7]],
        [[2,4],[3,5]],[[4,7],[5,6]]},[1,2,4]:
cor2:={[[2, 4], [3, 5]], [[4, 6], [5, 7]], [[1, 2, 4, 5, 7, 3, 6]], 
        [[4, 5], [6, 7]], [[2, 7, 3, 6], [4, 5]]},[1,2,4]:
car1:=`Canon/schreier-sims`(s):
if cor1<>car1 and cor2<>car1 then 
ERROR(`file Canon`,`error n. 16-1`,cor1,car1) 
fi;

# further tests are difficult due to non-uniquiness of the sgs


#
# `Canon/Canonical/Tensor`
#

# Symmetric metric
CanonMetricSymmetry:=1:

cor1 := -R[a,b]:
car1 := `Canon/Canonical/Tensor`(R[b,a],{[-1,[[1,2]]]},[1]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-1`,cor1,car1) 
fi:

cor1 := R[a,b]:
car1 := `Canon/Canonical/Tensor`(R[b,a],{[[1,2]]},[1]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-2`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical/Tensor`(R[a,-a,b,c],{[-1,[[1,2]]],[-1,[[3,4]]]},[1,3]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-3`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical/Tensor`(R[a,-a,b,c],{[[1,3],[2,4]],[-1,[[3,4]]]},[1,3]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-4`,cor1,car1) 
fi:

# Antisymmetric Metric
CanonMetricSymmetry:=-1:

cor1 := R[a,b]:
car1 := `Canon/Canonical/Tensor`(R[b,a],{[[1,2]]},[1]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-5`,cor1,car1) 
fi:

cor1 := -R[a,-a,b,c]:
car1 := `Canon/Canonical/Tensor`(R[a,-a,c,b],{[-1,[[1,2]]],[-1,[[3,4]]]},[1,3]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-6`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical/Tensor`(R[a,-a,b,c],{[[1,2]],[-1,[[3,4]]]},[1,3]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-7`,cor1,car1) 
fi:

# Metric without any symmetry
CanonMetricSymmetry:=0:

cor1 := R[a,-a,b,c]:
car1 := `Canon/Canonical/Tensor`(R[a,-a,b,c],{[-1,[[1,2]]],[-1,[[3,4]]]},[1,3]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 20-8`,cor1,car1) 
fi:

#
# `Canon/Canonical`
#

# Symmetric metric
CanonMetricSymmetry:=1:

cor1 := 0:
car1 := `Canon/Canonical`(`Canon/Canonical`(T[i,a]*T[j,-a]-T[i,-b]*T[j,b],
           CanonMetricSymmetry),CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-0`,cor1,car1) 
fi:

Canon[CanonDefine](R,4,{[-1,[[1,2]]],[[1,3],[2,4]]},[1,2,3,4]):
Canon[CanonDefine](T,3,{[[1,2]],[[2,3]]},[1,2,3]):

cor1 := 0:
car1 := `Canon/Canonical`(R[a,-a,b,c],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-1`,cor1,car1) 
fi:

cor1 := -R[i, a, j, b]*R[-a, c, d, e]*R[-b, -d, -c, -e]:
car1 := `Canon/Canonical`(R[-e, j, i, b]*R[-b, -c, a, d]*R[-d, c, -a, e],
           CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-2`,cor1,car1) 
fi:

cor1 := T[a,b,c]*T[-d,e,f]*T[-b,-c,-e]*T[-f,g,-g]+R[a,b,c,e]*T[-d,-c,f]*T[-b,-e,-f] :
car1 := eval(subs('_SCALAR'=proc(x) x end,
   `Canon/Canonical`(T[a,-b,e]*T[-c,b,-e]*T[f,c,-d]*T[-f,g,-g]+
      R[a,b,c,f]*T[-c,-d,-e]*T[-f,-b,e],CanonMetricSymmetry))):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-3`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(T[e,i,j]*R[-i,-j,-a,-b]*R[k,-y,-x,-k]*T[d,-d,-f],
            CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-4`,cor1,car1) 
fi:

# Antisymmetric Metric
CanonMetricSymmetry:=-1:

cor1 := -R[i,a,j,b]*R[-a,c,d,e]*R[-b,-d,-c,-e]:
car1 := `Canon/Canonical`(R[-e, j, i, b]*R[-b, -c, a, d]*R[-d, c, -a, e],
            CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-5`,cor1,car1) 
fi:

# Metric without any symmetry
CanonMetricSymmetry:=0:

cor1 := -R[i,a,j,-d]*R[-a,-e,b,c]*R[d,-b,e,-c]:  #-R[i,a,j,-b]*R[-a,-c,d,e]*R[b,-d,c,-e]:
car1 := `Canon/Canonical`(R[-e, j, i, b]*R[-b, -c, a, d]*R[-d, c, -a, e],
            CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 21-6`,cor1,car1) 
fi:


CanonMetricSymmetry:=1:
Canon[CanonDefine](R,4,{[-1,[[3,4]]],[[1,3],[2,4]]}):
Canon[CanonDefine](T,2,{[[1,2]]}):
Canon[CanonDefine](T,3,{[[1,2]],[[2,3]]}):

cor1 := 0:
car1 := `Canon/Canonical`(R[-a,-b,i,j]*T[e,-i,-j]*T[-f,d,-d]*R[-x,k,-y,-k],
             CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-1`,cor1,car1) 
fi:

Canon[CanonDefine](S,2,Symmetric):
cor1 := 0:
car1 := Canonical(S[ii2,ii1]-S[ii1,ii2]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-2`,cor1,car1) 
fi:

Canon[CanonDefine](S,2,AntiSymmetric):
cor1 := 0:
car1 := Canonical(S[ii2,ii1]+S[ii1,ii2]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-3`,cor1,car1) 
fi:


cor1 := 0:
car1 := `Canon/Canonical`(R[a,b,-a,-b]*R[c,d,-c,-d]*R[e,f,-e,-f]*
R[g,h,-g,-h]*R[i,j,-i,-j]*R[k,l,-k,-l]*R[m,n,-m,-n]-R[a1,b1,-a1,-b1]^7,
CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-4`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(R[a,b,d,-a]*R[c,-b,-c,f]*
R[e,-d,-e,-f]*R[g,h,-g,j]*R[i,-h,-i,l]*R[k,-j,-k,-l]-
R[g,h,-g,j]*R[c,-h,-c,l]*R[e,-j,-e,-l]*R[a,b,-a,d]*R[i,-b,-i,f]*R[k,-d,-f,-k],
CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-5`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(R[a,b,c,d]*R[e,f,k,h]*R[i,-a,-j,-e]*R[-b,-c,-d,-i]*R[-f,-k,-h,j],
CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-6`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(-R[i,-i,j,-j]^2,CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-7`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(A[b,e]*T[-b,-e]-A[e,b]*T[-b,-e],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-8`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(A[]*A[]*R[-d2,-d3,d1,d4]*R[-d5,b,a,d2]*R[-d4,d3,-d1,d5]+
R[b,d2,a,-d8]*R[d4,d5,d8,d3]*R[-d2,-d4,-d3,-d5]*A[]^2,CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-9`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(B[q,-q]*V[i,-i]^2*W[a,-a]^3*
W[-b,b]+V[i,-i]*V[j,-j]*W[-b1,b1]*W[-b2,b2]*W[-b3,b3]*W[-b4,b4]*B[b,-b]-
2*B[a,-a]*W[b3,-b3]^4*V[b,-b]^2,CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-10`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(V[i,-i]^2-V[j,-j]^2,CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-11`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(V[i,-i]^2-V[i,-i]*V[j,-j],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-12`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(V[j,-j,-z,z]-W[c,-c,a]*W[-a,b,-b]+V[-k]*V[k]-
(V[a,-a,b,-b]-W[a,-a,b]*W[-b,c,-c]+V[-a]*V[a]),CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-13`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(V[j]*V[-j]-W[a]*W[-a]+V[-k]*V[k]-2*V[-z]*V[z]+W[b]*W[-b],
             CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-14`,cor1,car1) 
fi:

cor1 := V[k]:
car1 := `Canon/Canonical`(V[j]*V[-j]*W[a]*W[-a]*A[k]+V[k]-
V[a]*V[-a]*W[j]*W[-j]*A[k],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-15`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(W[i,-j]*V[-i,j]-W[j,-i]*V[-j,i],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-16`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(V[j,-j]-V[-k,k],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-17`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(V[a,b]*W[-a,-b]*A[c,d]*B[-c,-d]-A[a,b]*B[-a,-b]*V[c,d]*W[-c,-d],
           CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-18`,cor1,car1) 
fi:

cor1 := 0:
car1 := `Canon/Canonical`(R[b,-c,a,-d]*R[-a,f,g,h]+R[b,-c,-d,z]*R[g,h,-z,f],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-19`,cor1,car1) 
fi:

cor1 := 0:
car1 := CanonDefine(Z, 4, {[[1,2]],[-1,[[3,4]]],[[1,3],[2,4]]}):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 22-20`,cor1,car1) 
fi:

CanonMetricSymmetry:=-1:
cor1 := 0:
car1 := `Canon/Canonical`(S[i,-i]+S[-k,k],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 23-1`,cor1,car1) 
fi:

CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(A[a2,-a2,b2,-b2]*A[a1,b1,-a1,-b1]-A[a1,b1,-a1,-b1]*A[a,-a,b,-b],
             CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 24-1`,cor1,car1) 
fi:


CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(R[m,-a,n,-b]*g[a,b]*R[k,i,l,j]*g[-i,-j]-
            R[k,a,l,b]*g[-a,-b]*R[m,i,n,j]*g[-i,-j],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-1`,cor1,car1) 
fi:


CanonMetricSymmetry:=-1:
cor1 := 0:
car1 := `Canon/Canonical`(R[m,-a,n,b]*g[a,-b]*R[k,i,l,j]*g[-i,-j]+
            R[k,a,l,b]*g[-a,-b]*R[m,i,n,j]*g[-i,-j],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-2`,cor1,car1) 
fi:


CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(R[m,b,-m,a]*g[-b,-a]*R[k,i,-k,j]*g[-i,-j]-
          R[k,b,-k,a]*g[-b,-a]*R[m,i,-m,j]*g[-i,-j],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-3`,cor1,car1) 
fi:

CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(T[-j,j]*A[a,b]*A[-a,-b]*A[c,d]*A[-d,-c]-
T[n,-n]*A[a,b]*A[-b,-a]*A[c,d]*A[-c,-d],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-4`,cor1,car1) 
fi:

CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(R[a,b,c,d]*R[-a,-b,-c,-d]*
R[a1,b1,c1,d1]*R[-a1,-b1,-c1,-d1]*R[a2,b2,c2,d2]*R[-a2,-b2,-c2,-d2]-
R[a3,b3,c3,d3]*R[-a3,-b3,-c3,-d3]*
R[a14,b14,c14,d14]*R[-a14,-b14,-c14,-d14]*R[d2,c2,a2,b2]*R[-a2,-b2,-d2,-c2],
CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-5`,cor1,car1) 
fi:

CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(A[i,-a,-b,-c,-d]*B[j,a,b]*C[c,d]-
C[a,b]*A[i,-c,-d,-a,-b]*B[j,c,d],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-6`,cor1,car1) 
fi:

Canon[CanonDefine](g,2,{[[1,2]]}):
CanonMetricSymmetry:=1:
cor1 := 0:
car1 := `Canon/Canonical`(A[i,-a,-b,-c]*A[j,a,b,-d]*g[c,d]-
A[j,-a,-b,-c]*A[i,a,b,-d]*g[c,d],CanonMetricSymmetry):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 25-7`,cor1,car1) 
fi:


unassign(seq(evaln(p||i),i=1..20));
unassign(seq(evaln(i||i),i=1..20));
unassign('s');

CanonMetricSymmetry:=1:
cor1 := 0:
car1 := Canon[Canonical](10*Z[]*qqq*A[-q1,q1]*B[-b,b]^2*C[c,d]*C[-c,e]*
D[-e,-d]*E[f,g]*E[-f,h]*F[-h,-g]*G[i]*H[j,-z1,z1]*J[l,-m,-n]*K[m]*L[-o,n]*
M[l1,-m1,-n1]*N[m1]*S[-o1,n1]*C[c1,d1]*C[-c1,-e1]*D[e1,-d1]*_SCALAR(P[r]*
Q[-r,-s]*Q[s])^3*P[r1]*Q[-r1,-s1]*Q[s1]-10*qqq*G[i]*E[a,b]*E[-a,c]*F[-b,-c]*
_SCALAR(C[c1,d]*C[-c1,d1]*D[-d,-d1])^2*_SCALAR(P[e]*Q[e1]*Q[-e,-e1])^4*
_SCALAR(B[f,-f])^2*A[g,-g]*Z[]*H[j,h,-h]*J[l,m,m1]*K[-m]*L[-o,-m1]*
M[l1,n,n1]*N[-n]*S[-o1,-n1]):
if cor1<>car1 then 
ERROR(`file Canon`,`error n. 26-1`,cor1,car1) 
fi:





#
# clear all variables
# must be the last commands
#

unassign('s');
unassign(seq(evaln(cor||i),i=1..20));
unassign(seq(evaln(car||i),i=1..20));
unassign(seq(evaln(p||i),i=1..20));
unassign(seq(evaln(K||i),i=1..20));
unassign(seq(evaln(B||i),i=1..20));
unassign(seq(evaln(i||i),i=1..20));
unassign(seq(evaln(sch||i),i=1..20));
unassign(seq(evaln(bac||i),i=1..20));
CanonTable := table([]):
CanonMetricSymmetry:=1:


